 /*********************************************************************/
 /*                                                                   */
 /*  Source File Name: db2AuCfg                                       */
 /*                                                                   */
 /*  (C) COPYRIGHT International Business Machines Corp. 1987, 1999   */
 /*  All Rights Reserved                                              */
 /*  Licensed Materials - Property of IBM                             */
 /*                                                                   */
 /*  US Government Users Restricted Rights - Use, duplication or      */
 /*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.*/
 /*                                                                   */
 /*  Function = Include File defining:                                */
 /*               db2AutoConfig and corresponding data structures     */
 /*                                                                   */
 /*********************************************************************/


 %dcl SQL_H_DB2AUCFG char ext;
 %if SQL_H_DB2AUCFG = '' %then %do;
 %SQL_H_DB2AUCFG = 'Y';

 %dcl SQL_H_SQLENV char ext;
 %if SQL_H_SQLENV = '' %then %do;
    %include sqlenv;
 %end;
 %dcl SQL_H_DB2APIDF char ext;
 %if SQL_H_DB2APIDF = '' %then %do;
    %include db2ApiDf;
 %end;

 /* type definitions for the smartguide */
 define structure
   1 db2AutoConfigElement,
     2 token type db2int32,
     2 value type db2int32;

 define structure
   1 db2AutoConfigArray,
     2 numElements type db2Uint32,        /* number of array elements */
     2 pElements   handle db2AutoConfigElement;  /* the element array */

 define alias db2AutoConfigInput type db2AutoConfigArray;
 define alias db2AutoConfigDiags type db2AutoConfigArray;

 define structure
   1 db2ConfigValues,
     2 numElements type db2Uint32,
     2 pConfigs    handle sqlfupd,
     2 pDataArea   pointer;

 define structure
   1 db2AutoConfigOutput,
     2 oOldDbValues  type db2ConfigValues,
                     /* current/old database configurations */
     2 oOldDbmValues type db2ConfigValues,
                     /* current/old database manager configurations */
     2 oNewDbValues  type db2ConfigValues,
                     /* suggested/new database configurations */
     2 oNewDbmValues type db2ConfigValues,
                     /* suggested/new database manager configurations */
     2 oDiagnostics  type db2AutoConfigDiags; /* diagnostic details */

 dcl DB2_SG_PROD_VERSION_SIZE fixed bin(31) value(16);

 define structure
   1 db2AutoConfigInterface,
     2 iProductID      type db2int32,/* product id */
     2 iProductVersion char (DB2_SG_PROD_VERSION_SIZE) varz,
                       /* product version */
     2 iDbAlias        char (SQL_ALIAS_SZ) varz,/* database alias */
     2 iApply          type db2int32,
                       /* update the configuration automatically */
     2 iParams         type db2AutoConfigInput,
                       /* input parameters to the smart guide */
     2 oResult         type db2AutoConfigOutput;
                       /* results from the smart guide */

 /* Tokens for the SmartGuide input parameters */
 dcl DB2_SG_MEMORY_PERCENTAGE  fixed bin(31) value(1);
     /* The percentage of the memory the database manager trys to use */
 dcl DB2_SG_WORKLOAD           fixed bin(31) value(2);
     /* The type of workload that best reflects the database. */
 dcl DB2_SG_NUM_STATEMENT      fixed bin(31) value(3);
    /* The average number of SQL statements in a single unit of work. */
 dcl DB2_SG_TRANS_PER_MINUTE   fixed bin(31) value(4);
     /* The number of transactions per minute. */
 dcl DB2_SG_ADMIN_PRIORITY     fixed bin(31) value(5);
     /* The database administration priority */
 dcl DB2_SG_IS_POPULATED       fixed bin(31) value(6);
     /* Is the database populated with data */
 dcl DB2_SG_LOCAL_APPLICATION  fixed bin(31) value(7);
     /* Average number of connected local application */
 dcl DB2_SG_REMOTE_APPLICATION fixed bin(31) value(8);
     /* Average number of connected remote application */
 dcl DB2_SG_ISOLATION_LEVEL    fixed bin(31) value(9);
     /* Isolation level that best reflects your applications */

 /* Valid values for the SmartGuide input parameters */


 dcl DB2_SG_MEMORY_PERCENTAGE_MIN            fixed bin(31) value(1);
 dcl DB2_SG_MEMORY_PERCENTAGE_MAX            fixed bin(31) value(100);


 dcl DB2_SG_WORKLOAD_MIN                     fixed bin(31) value(1);
 dcl DB2_SG_WORKLOAD_QUERIES                 fixed bin(31) value(1);
 dcl DB2_SG_WORKLOAD_MIXED                   fixed bin(31) value(2);
 dcl DB2_SG_WORKLOAD_TRANSACTIONS            fixed bin(31) value(3);
 dcl DB2_SG_WORKLOAD_MAX                     fixed bin(31) value(3);

 dcl DB2_SG_ADMIN_PRIORITY_MIN               fixed bin(31) value(1);
 dcl DB2_SG_ADMIN_PRIORITY_TRANSACTION       fixed bin(31) value(1);
 dcl DB2_SG_ADMIN_PRIORITY_BOTH              fixed bin(31) value(2);
 dcl DB2_SG_ADMIN_PRIORITY_RECOVERY          fixed bin(31) value(3);
 dcl DB2_SG_ADMIN_PRIORITY_MAX               fixed bin(31) value(3);


 dcl DB2_SG_TRANS_PER_MINUTE_MIN             fixed bin(31) value(1);
 dcl DB2_SG_TRANS_PER_MINUTE_MAX             fixed bin(31) value(50000);


 dcl DB2_SG_IS_POPULATED_MIN                 fixed bin(31) value(1);
 dcl DB2_SG_IS_POPULATED_YES                 fixed bin(31) value(1);
 dcl DB2_SG_IS_POPULATED_NO                  fixed bin(31) value(2);
 dcl DB2_SG_IS_POPULATED_MAX                 fixed bin(31) value(2);


 dcl DB2_SG_LOCAL_APPLICATION_MIN            fixed bin(31) value(0);
 dcl DB2_SG_LOCAL_APPLICATION_MAX            fixed bin(31) value(5000);


 dcl DB2_SG_REMOTE_APPLICATION_MIN           fixed bin(31) value(0);
 dcl DB2_SG_REMOTE_APPLICATION_MAX           fixed bin(31) value(5000);


 dcl DB2_SG_ISOLATION_LEVEL_MIN              fixed bin(31) value(1);
 dcl DB2_SG_ISOLATION_LEVEL_REPEAT_READ      fixed bin(31) value(1);
 dcl DB2_SG_ISOLATION_LEVEL_READ_STABILITY   fixed bin(31) value(2);
 dcl DB2_SG_ISOLATION_LEVEL_CURSOR_STABILITY fixed bin(31) value(3);
 dcl DB2_SG_ISOLATION_LEVEL_UNCOMMITTED_READ fixed bin(31) value(4);
 dcl DB2_SG_ISOLATION_LEVEL_MAX              fixed bin(31) value(4);



 /* Tokens for SmartGuide diagnostics :                               */
 /* These token/values pairs may appear in                            */
 /* db2AutoConfigOutput.db2AutoConfigDiags.                           */
 /* Some of tokens return a meaningful value, and some are just flags */
 /* indicating a state (for which the value will usually be -1, but   */
 /* should be ignored in any event).                                  */
 /* DB2_SG_DIAG_NO_VALUE          - the "no value" value for flag     */
 /*                                 type diagnostic tokens            */
 /* DB2_SG_DIAG_DEFAULT_USED      - flag indicating the default       */
 /*                                 SmartGuide is returning this      */
 /*                                 result set                        */
 /* DB2_SG_DIAG_NOT_ENOUGH_MEMORY - value returned is the recommended */
 /*                                 amount of memory for given inputs */
 /* DB2_SG_DIAG_NOT_ENOUGH_DISKS  - value returned is the recommended */
 /*                                 # of disks for given inputs       */
 /* DB2_SG_DIAG_LOW_MEMORY_PERCENTAGE - value returned is the %age of */
 /*                                 memory to expect DB2 to consume   */
 /*                                 for given inputs                  */
 /* DB2_SG_DIAG_DBA_MEMORY        - value returned is the actual      */
 /*                                 amount of physical memory on the  */
 /*                                 target machine, and accompanies   */
 /*                                 other memory diagnostics          */
 /* DB2_SG_DIAG_DBA_BUFFER_POOL_SIZE - flag indicating that the buffer*/
 /*                                 pools could not be increased due  */
 /*                                 to memory constraints             */
 /* DB2_SG_DIAG_DBA_TRANSACTION_RATE - flag indicating that the       */
 /*                                 requested transaction rate is     */
 /*                                 optimistic                        */

 dcl DB2_SG_DIAG_NO_VALUE              fixed bin(31) value(-1);
 dcl DB2_SG_DIAG_DEFAULT_USED          fixed bin(31) value(0);
 dcl DB2_SG_DIAG_NOT_ENOUGH_MEMORY     fixed bin(31) value(-1);
 dcl DB2_SG_DIAG_NOT_ENOUGH_DISKS      fixed bin(31) value(-2);
 dcl DB2_SG_DIAG_LOW_MEMORY_PERCENTAGE fixed bin(31) value(-3);
 dcl DB2_SG_DIAG_DBA_MEMORY            fixed bin(31) value(1100);
     /* See associated DBA1100 message */
 dcl DB2_SG_DIAG_DBA_BUFFER_POOL_SIZE  fixed bin(31) value(1108);
     /* See associated DBA1108 message */
 dcl DB2_SG_DIAG_DBA_TRANSACTION_RATE  fixed bin(31) value(1109);
     /* See associated DBA1109 message */


 /* Product ID for the product using the Smart Guide */
 dcl DB2_SG_PID_DEFAULT                   fixed bin(31) value(0);
 dcl DB2_SG_PID_WEBSPHERE_COMMERCE_SUITE  fixed bin(31) value(1);
 dcl DB2_SG_PID_SAP                       fixed bin(31) value(2);
 dcl DB2_SG_PID_WEBSPHERE_ADVANCED_SERVER fixed bin(31) value(3);
 dcl DB2_SG_PID_SIEBEL                    fixed bin(31) value(4);


 /* Indicate to apply the smartguide recommendation or not */
 dcl DB2_SG_NOT_APPLY fixed bin(31) value(0);
 dcl DB2_SG_APPLY     fixed bin(31) value(1);


 /* Smartguide return codes */
 dcl DB2_SG_RC_OK     fixed bin(31) value(0);
 dcl DB2_SG_RC_FAIL   fixed bin(31) value(-1);

 dcl db2AutoConfig
     entry (type db2Uint32, /* versions defined in db2ApiDf.cpy */
            pointer,        /* pointer to db2AutoConfigInterface */
            pointer)        /* SQLCA */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external ('db2AutoConfig');

 dcl db2AutoConfigFreeMemory
      entry (type db2Uint32,
             pointer,
             pointer)        /* SQLCA */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2AutoConfigFreeMemory');

 %end; /* DB2AUCFG_H */
