 /* ++                                                                */
 /*                                                                   */
 /* Copyright (C) 1992-1999 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   snmp.cpy                                                        */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Definitions for SNMP development.                               */
 /*                                                                   */
 /* --                                                                */

 %dcl #INC_SNMP char ext;
 %if #INC_SNMP = '' %then %do;
 %#INC_SNMP = 'Y';

 /*******************************************************************/
 /*                                                                 */
 /* Additional Header Files                                         */
 /*                                                                 */
 /*******************************************************************/

 %dcl #WINDEF_ char ext;
 %if #WINDEF_ = '' %then %do;
 %include windef;
 %end;


 /*******************************************************************/
 /*                                                                 */
 /* SNMP Type Definitions                                           */
 /*                                                                 */
 /*******************************************************************/

 define structure
   1 AsnOctetString,
     2 stream  pointer,
     2 length  type UINT,
     2 dynamic type BOOL;

 define structure
   1 AsnObjectIdentifier,
     2 idLength type UINT,
     2 ids      pointer;

 define alias AsnInteger32        type LONG;
 define alias AsnUnsigned32       type ULONG;
 define alias AsnCounter64        type ULARGE_INTEGER;
 define alias AsnCounter32        type AsnUnsigned32;
 define alias AsnGauge32          type AsnUnsigned32;
 define alias AsnTimeticks        type AsnUnsigned32;
 define alias AsnBits             type AsnOctetString;
 define alias AsnSequence         type AsnOctetString;
 define alias AsnImplicitSequence type AsnOctetString;
 define alias AsnIPAddress        type AsnOctetString;
 define alias AsnNetworkAddress   type AsnOctetString;
 define alias AsnDisplayString    type AsnOctetString;
 define alias AsnOpaque           type AsnOctetString;

 define structure
   1 AsnAny,
     2 asnType      type BYTE,
     2 asnValue union,
       3 number     type AsnInteger32,       /* ASN_INTEGER32        */
       3 unsigned32 type AsnUnsigned32,      /* ASN_UNSIGNED32       */
       3 counter64  type AsnCounter64,       /* ASN_COUNTER64        */
       3 string     type AsnOctetString,     /* ASN_OCTETSTRING      */
       3 bits       type AsnBits,            /* ASN_BITS             */
       3 object     type AsnObjectIdentifier,/* ASN_OBJECTIDENTIFIER */
       3 sequence   type AsnSequence,        /* ASN_SEQUENCE         */
       3 address    type AsnIPAddress,       /* ASN_IPADDRESS        */
       3 counter    type AsnCounter32,       /* ASN_COUNTER32        */
       3 gauge      type AsnGauge32,         /* ASN_GAUGE32          */
       3 ticks      type AsnTimeticks,       /* ASN_TIMETICKS        */
       3 arbitrary  type AsnOpaque;          /* ASN_OPAQUE           */

 define alias AsnObjectName   type AsnObjectIdentifier;
 define alias AsnObjectSyntax type AsnAny;

 define structure
   1 SnmpVarBind,
     2 name  type AsnObjectName,
     2 value type AsnObjectSyntax;

 define structure
   1 SnmpVarBindList,
     2 list  handle SnmpVarBind,
     2 len   type UINT;


 %dcl #INC_WINSNMP char ext;
 %if #INC_WINSNMP = '' %then %do;

 /*******************************************************************/
 /*                                                                 */
 /* ASN/BER Base Types                                              */
 /*                                                                 */
 /*******************************************************************/

 dcl ASN_UNIVERSAL   type BYTE value('00'xn);
 dcl ASN_APPLICATION type BYTE value('40'xn);
 dcl ASN_CONTEXT     type BYTE value('80'xn);
 dcl ASN_PRIVATE     type BYTE value('C0'xn);

 dcl ASN_PRIMITIVE   type BYTE value('00'xn);
 dcl ASN_CONSTRUCTOR type BYTE value('20'xn);

 /*******************************************************************/
 /*                                                                 */
 /* PDU Type Values                                                 */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_PDU_GET         type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '0'xn));
 dcl SNMP_PDU_GETNEXT     type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '1'xn));
 dcl SNMP_PDU_RESPONSE    type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '2'xn));
 dcl SNMP_PDU_SET         type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '3'xn));
 dcl SNMP_PDU_V1TRAP      type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '4'xn));
 dcl SNMP_PDU_GETBULK     type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '5'xn));
 dcl SNMP_PDU_INFORM      type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '6'xn));
 dcl SNMP_PDU_TRAP        type BYTE
     value (ior (ASN_CONTEXT, ASN_CONSTRUCTOR, '7'xn));

 %end; /* _INC_WINSNMP */

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Simple Syntax Values                                       */
 /*                                                                 */
 /*******************************************************************/

 dcl ASN_INTEGER          type BYTE
     value (ior (ASN_UNIVERSAL, ASN_PRIMITIVE, '02'xn));
 dcl ASN_BITS             type BYTE
     value (ior (ASN_UNIVERSAL, ASN_PRIMITIVE, '03'xn));
 dcl ASN_OCTETSTRING      type BYTE
     value (ior (ASN_UNIVERSAL, ASN_PRIMITIVE, '04'xn));
 dcl ASN_NULL             type BYTE
     value (ior (ASN_UNIVERSAL, ASN_PRIMITIVE, '05'xn));
 dcl ASN_OBJECTIDENTIFIER type BYTE
     value (ior (ASN_UNIVERSAL, ASN_PRIMITIVE, '06'xn));

 dcl ASN_INTEGER32        type BYTE value (ASN_INTEGER);

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Constructor Syntax Values                                  */
 /*                                                                 */
 /*******************************************************************/

 dcl ASN_SEQUENCE         type BYTE
     value (ior (ASN_UNIVERSAL, ASN_CONSTRUCTOR, '10'xn));

 dcl ASN_SEQUENCEOF       type BYTE value(ASN_SEQUENCE);

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Application Syntax Values                                  */
 /*                                                                 */
 /*******************************************************************/

 dcl ASN_IPADDRESS        type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '00'xn));
 dcl ASN_COUNTER32        type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '01'xn));
 dcl ASN_GAUGE32          type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '02'xn));
 dcl ASN_TIMETICKS        type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '03'xn));
 dcl ASN_OPAQUE           type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '04'xn));
 dcl ASN_COUNTER64        type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '06'xn));
 dcl ASN_UNSIGNED32       type BYTE
     value (ior (ASN_APPLICATION, ASN_PRIMITIVE, '07'xn));


 /*******************************************************************/
 /*                                                                 */
 /* SNMP Exception Conditions                                       */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_EXCEPTION_NOSUCHOBJECT   type BYTE
     value (ior (ASN_CONTEXT, ASN_PRIMITIVE, '00'xn));
 dcl SNMP_EXCEPTION_NOSUCHINSTANCE type BYTE
     value (ior (ASN_CONTEXT, ASN_PRIMITIVE, '01'xn));
 dcl SNMP_EXCEPTION_ENDOFMIBVIEW   type BYTE
     value (ior (ASN_CONTEXT, ASN_PRIMITIVE, '02'xn));


 /*******************************************************************/
 /*                                                                 */
 /* SNMP Request Types (used in SnmpExtensionQueryEx)               */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_EXTENSION_GET          type BYTE
     value (SNMP_PDU_GET);
 dcl SNMP_EXTENSION_GET_NEXT     type BYTE
     value (SNMP_PDU_GETNEXT);
 dcl SNMP_EXTENSION_GET_BULK     type BYTE
     value (SNMP_PDU_GETBULK);
 dcl SNMP_EXTENSION_SET_TEST     type BYTE
     value (ior (ASN_PRIVATE, ASN_CONSTRUCTOR, '0'xn));
 dcl SNMP_EXTENSION_SET_COMMIT   type BYTE
     value (SNMP_PDU_SET);
 dcl SNMP_EXTENSION_SET_UNDO     type BYTE
     value (ior (ASN_PRIVATE, ASN_CONSTRUCTOR, '1'xn));
 dcl SNMP_EXTENSION_SET_CLEANUP  type BYTE
     value (ior (ASN_PRIVATE, ASN_CONSTRUCTOR, '2'xn));


 /*******************************************************************/
 /*                                                                 */
 /* SNMP Error Codes                                                */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_ERRORSTATUS_NOERROR             fixed bin(31) value(0);
 dcl SNMP_ERRORSTATUS_TOOBIG              fixed bin(31) value(1);
 dcl SNMP_ERRORSTATUS_NOSUCHNAME          fixed bin(31) value(2);
 dcl SNMP_ERRORSTATUS_BADVALUE            fixed bin(31) value(3);
 dcl SNMP_ERRORSTATUS_READONLY            fixed bin(31) value(4);
 dcl SNMP_ERRORSTATUS_GENERR              fixed bin(31) value(5);
 dcl SNMP_ERRORSTATUS_NOACCESS            fixed bin(31) value(6);
 dcl SNMP_ERRORSTATUS_WRONGTYPE           fixed bin(31) value(7);
 dcl SNMP_ERRORSTATUS_WRONGLENGTH         fixed bin(31) value(8);
 dcl SNMP_ERRORSTATUS_WRONGENCODING       fixed bin(31) value(9);
 dcl SNMP_ERRORSTATUS_WRONGVALUE          fixed bin(31) value(10);
 dcl SNMP_ERRORSTATUS_NOCREATION          fixed bin(31) value(11);
 dcl SNMP_ERRORSTATUS_INCONSISTENTVALUE   fixed bin(31) value(12);
 dcl SNMP_ERRORSTATUS_RESOURCEUNAVAILABLE fixed bin(31) value(13);
 dcl SNMP_ERRORSTATUS_COMMITFAILED        fixed bin(31) value(14);
 dcl SNMP_ERRORSTATUS_UNDOFAILED          fixed bin(31) value(15);
 dcl SNMP_ERRORSTATUS_AUTHORIZATIONERROR  fixed bin(31) value(16);
 dcl SNMP_ERRORSTATUS_NOTWRITABLE         fixed bin(31) value(17);
 dcl SNMP_ERRORSTATUS_INCONSISTENTNAME    fixed bin(31) value(18);

 /*******************************************************************/
 /*                                                                 */
 /* SNMPv1 Trap Types                                               */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_GENERICTRAP_COLDSTART           fixed bin(31) value(0);
 dcl SNMP_GENERICTRAP_WARMSTART           fixed bin(31) value(1);
 dcl SNMP_GENERICTRAP_LINKDOWN            fixed bin(31) value(2);
 dcl SNMP_GENERICTRAP_LINKUP              fixed bin(31) value(3);
 dcl SNMP_GENERICTRAP_AUTHFAILURE         fixed bin(31) value(4);
 dcl SNMP_GENERICTRAP_EGPNEIGHLOSS        fixed bin(31) value(5);
 dcl SNMP_GENERICTRAP_ENTERSPECIFIC       fixed bin(31) value(6);

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Access Types                                               */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_ACCESS_NONE                     fixed bin(31) value(0);
 dcl SNMP_ACCESS_NOTIFY                   fixed bin(31) value(1);
 dcl SNMP_ACCESS_READ_ONLY                fixed bin(31) value(2);
 dcl SNMP_ACCESS_READ_WRITE               fixed bin(31) value(3);
 dcl SNMP_ACCESS_READ_CREATE              fixed bin(31) value(4);

 /*******************************************************************/
 /*                                                                 */
 /* SNMP API Return Code Definitions                                */
 /*                                                                 */
 /*******************************************************************/

 define alias SNMPAPI type INT;
 %dcl SNMP_FUNC_TYPE char ext;
 %SNMP_FUNC_TYPE = WINAPI;

 dcl SNMPAPI_NOERROR fixed bin (31) value (TRUE);
 dcl SNMPAPI_ERROR   fixed bin (31) value (FALSE);

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Extension API Prototypes                                   */
 /*                                                                 */
 /*******************************************************************/

 dcl SnmpExtensionInit
     entry (type DWORD,
            type HANDLE byaddr,
            type AsnObjectIdentifier byaddr)
     returns (type BOOL byvalue optional)
     SNMP_FUNC_TYPE external('SnmpExtensionInit');

 dcl SnmpExtensionInitEx
     entry (type AsnObjectIdentifier byaddr)
     returns (type BOOL byvalue optional)
     SNMP_FUNC_TYPE external('SnmpExtensionInitEx');

 dcl SnmpExtensionMonitor
     entry (type LPVOID)
     returns (type BOOL byvalue optional)
     SNMP_FUNC_TYPE external('SnmpExtensionMonitor');

 dcl SnmpExtensionQuery
     entry (type BYTE,
            type SnmpVarBindList byaddr,
            type AsnInteger32 byaddr,
            type AsnInteger32 byaddr)
     returns (type BOOL byvalue optional)
     SNMP_FUNC_TYPE external('SnmpExtensionQuery');

 dcl SnmpExtensionQueryEx
     entry (type UINT,
            type UINT,
            type SnmpVarBindList byaddr,
            type AsnOctetString byaddr,
            type AsnInteger32 byaddr,
            type AsnInteger32 byaddr)
     returns (type BOOL byvalue optional)
     SNMP_FUNC_TYPE external('SnmpExtensionQueryEx');

 dcl SnmpExtensionTrap
     entry (type AsnObjectIdentifier byaddr,
            type AsnInteger32 byaddr,
            type AsnInteger32 byaddr,
            type AsnTimeticks byaddr,
            type SnmpVarBindList byaddr)
     returns (type BOOL byvalue optional)
     SNMP_FUNC_TYPE external('SnmpExtensionTrap');

 dcl SnmpExtensionClose
     entry ( )
     SNMP_FUNC_TYPE external('SnmpExtensionClose');

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Extension API Type Definitions                             */
 /*                                                                 */
 /*******************************************************************/

 define alias PFNSNMPEXTENSIONINIT
        entry (type DWORD,
               type HANDLE byaddr,
               type AsnObjectIdentifier byaddr)
        returns (type BOOL byvalue optional)
        limited SNMP_FUNC_TYPE;

 define alias PFNSNMPEXTENSIONINITEX
        entry (type AsnObjectIdentifier byaddr)
        returns (type BOOL byvalue optional)
        limited SNMP_FUNC_TYPE;

 define alias PFNSNMPEXTENSIONMONITOR
        entry (type LPVOID)
        returns (type BOOL byvalue optional)
        limited SNMP_FUNC_TYPE;

 define alias PFNSNMPEXTENSIONQUERY
         entry (type BYTE,
                type SnmpVarBindList byaddr,
                type AsnInteger32 byaddr,
                type AsnInteger32 byaddr)
        returns (type BOOL byvalue optional)
        limited SNMP_FUNC_TYPE;

 define alias PFNSNMPEXTENSIONQUERYEX
        entry (type UINT,
               type UINT,
               type SnmpVarBindList byaddr,
               type AsnOctetString byaddr,
               type AsnInteger32 byaddr,
               type AsnInteger32 byaddr)
        returns (type BOOL byvalue optional)
        limited SNMP_FUNC_TYPE;

 define alias PFNSNMPEXTENSIONTRAP
        entry (type AsnObjectIdentifier byaddr,
               type AsnInteger32 byaddr,
               type AsnInteger32 byaddr,
               type AsnTimeticks byaddr,
               type SnmpVarBindList byaddr)
        returns (type BOOL byvalue optional)
        limited SNMP_FUNC_TYPE;

 define alias PFNSNMPEXTENSIONCLOSE
        entry ()
        limited SNMP_FUNC_TYPE ;

 /*******************************************************************/
 /*                                                                 */
 /* SNMP API Prototypes                                             */
 /*                                                                 */
 /*******************************************************************/

 dcl SnmpUtilOidCpy
     entry (type AsnObjectIdentifier byaddr,
            type AsnObjectIdentifier byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOidCpy');

 dcl SnmpUtilOidAppend
     entry (type AsnObjectIdentifier byaddr,
            type AsnObjectIdentifier byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOidAppend');

 dcl SnmpUtilOidNCmp
     entry (type AsnObjectIdentifier byaddr,
            type AsnObjectIdentifier byaddr,
            type UINT)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOidNCmp');

 dcl SnmpUtilOidCmp
     entry (type AsnObjectIdentifier byaddr,
            type AsnObjectIdentifier byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOidCmp');

 dcl SnmpUtilOidFree
     entry (type AsnObjectIdentifier byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilOidFree');

 dcl SnmpUtilOctetsCmp
     entry (type AsnOctetString byaddr,
            type AsnOctetString byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOctetsCmp');

 dcl SnmpUtilOctetsNCmp
     entry (type AsnOctetString byaddr,
            type AsnOctetString byaddr,
            type UINT)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOctetsNCmp');

 dcl SnmpUtilOctetsCpy
     entry (type AsnOctetString byaddr,
            type AsnOctetString byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOctetsCpy');

 dcl SnmpUtilOctetsFree
     entry (type AsnOctetString byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilOctetsFree');

 dcl SnmpUtilAsnAnyCpy
     entry (type AsnAny byaddr,
            type AsnAny byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilAsnAnyCpy');

 dcl SnmpUtilAsnAnyFree
     entry (type AsnAny byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilAsnAnyFree');

 dcl SnmpUtilVarBindCpy
     entry (type SnmpVarBind byaddr,
            type SnmpVarBind byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilVarBindCpy');

 dcl SnmpUtilVarBindFree
     entry (type SnmpVarBind byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilVarBindFree');

 dcl SnmpUtilVarBindListCpy
     entry (type SnmpVarBindList byaddr,
            type SnmpVarBindList byaddr)
     returns (type SNMPAPI byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilVarBindListCpy');

 dcl SnmpUtilVarBindListFree
     entry (type SnmpVarBindList byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilVarBindListFree');

 dcl SnmpUtilMemFree
     entry (type LPVOID)
     SNMP_FUNC_TYPE external('SnmpUtilMemFree');

 dcl SnmpUtilMemAlloc
     entry (type UINT)
     returns (type LPVOID byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilMemAlloc');

 dcl SnmpUtilMemReAlloc
     entry (type LPVOID,
            type UINT)
     returns (type LPVOID byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilMemReAlloc');

 dcl SnmpUtilOidToA
     entry (type AsnObjectIdentifier byaddr)
     returns (type LPSTR byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilOidToA');

 dcl SnmpUtilIdsToA
     entry ((*) type UINT byaddr,
            type UINT)
     returns (type LPSTR byvalue optional)
     SNMP_FUNC_TYPE external('SnmpUtilIdsToA');

 dcl SnmpUtilPrintOid
     entry (type AsnObjectIdentifier byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilPrintOid');

 dcl SnmpUtilPrintAsnAny
     entry (type AsnAny byaddr)
     SNMP_FUNC_TYPE external('SnmpUtilPrintAsnAny');

 dcl SnmpSvcGetUptime
     entry ()
     returns (type DWORD byvalue optional)
     SNMP_FUNC_TYPE external('SnmpSvcGetUptime');

 dcl SnmpSvcSetLogLevel
     entry (type INT)
     SNMP_FUNC_TYPE external('SnmpSvcSetLogLevel');

 dcl SnmpSvcSetLogType
     entry (type INT)
     SNMP_FUNC_TYPE external('SnmpSvcSetLogType');

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Debugging Definitions                                      */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_LOG_SILENT         fixed bin(15) value('0'xn);
 dcl SNMP_LOG_FATAL          fixed bin(15) value('1'xn);
 dcl SNMP_LOG_ERROR          fixed bin(15) value('2'xn);
 dcl SNMP_LOG_WARNING        fixed bin(15) value('3'xn);
 dcl SNMP_LOG_TRACE          fixed bin(15) value('4'xn);
 dcl SNMP_LOG_VERBOSE        fixed bin(15) value('5'xn);

 dcl SNMP_OUTPUT_TO_CONSOLE  fixed bin(15) value('1'xn);
 dcl SNMP_OUTPUT_TO_LOGFILE  fixed bin(15) value('2'xn);
 dcl SNMP_OUTPUT_TO_EVENTLOG fixed bin(15) value('4'xn);
     /* no longer supported */
 dcl SNMP_OUTPUT_TO_DEBUGGER fixed bin(15) value('8'xn);

 /*******************************************************************/
 /*                                                                 */
 /* SNMP Debugging Prototypes                                       */
 /*                                                                 */
 /*******************************************************************/

 dcl SnmpUtilDbgPrint
     entry (type INT, /* see log levels above... */
            char (*) varz byaddr list)
     WINAPIV external('SnmpUtilDbgPrint'); /* not SNMP_FUNC_TYPE!!! */

 %dcl DBG fixed ext;
 %SNMPDBG: proc (#x_) returns (char);
   dcl #x_ char;
   if DBG ^= 0 then
     return ('SnmpUtilDbgPrint(' || #x_ || ')');
   else
     return ('');
 %end SNMPDBG;
 %dcl SNMPDBG entry;

 /*******************************************************************/
 /*                                                                 */
 /* Miscellaneous definitions                                       */
 /*                                                                 */
 /*******************************************************************/

 %DEFINE_SIZEOF: proc (Array) returns (char);
   dcl Array char;
   return ('dim (' || Array || ', 1)');
 %end DEFINE_SIZEOF;
 %dcl DEFINE_SIZEOF entry;

 %DEFINE_OID: proc (SubIdArray) returns (char);
   dcl SubIdArray char;
   return (DEFINE_SIZEOF(SubIdArray) || ', addr(' || SubIdArray || ')');
 %end DEFINE_OID;
 %dcl DEFINE_OID entry;

 %DEFINE_NULLOID: proc () returns (char);
   return ('0, sysnull()');
 %end DEFINE_NULLOID;
 %dcl DEFINE_NULLOID entry;

 %DEFINE_NULLOCTETS: proc () returns (char);
   return ('sysnull(), 0, FALSE');
 %end DEFINE_NULLOCTETS;
 %dcl DEFINE_NULLOCTETS entry;


 dcl DEFAULT_SNMP_PORT_UDP     fixed bin(31) value(161);
 dcl DEFAULT_SNMP_PORT_IPX     fixed bin(31) value(36879);
 dcl DEFAULT_SNMPTRAP_PORT_UDP fixed bin(31) value(162);
 dcl DEFAULT_SNMPTRAP_PORT_IPX fixed bin(31) value(36880);

 dcl SNMP_MAX_OID_LEN          fixed bin(31) value(128);

 /*******************************************************************/
 /*                                                                 */
 /* API Error Code Definitions                                      */
 /*                                                                 */
 /*******************************************************************/

 dcl SNMP_MEM_ALLOC_ERROR          fixed bin(31) value(1);
 dcl SNMP_BERAPI_INVALID_LENGTH    fixed bin(31) value(10);
 dcl SNMP_BERAPI_INVALID_TAG       fixed bin(31) value(11);
 dcl SNMP_BERAPI_OVERFLOW          fixed bin(31) value(12);
 dcl SNMP_BERAPI_SHORT_BUFFER      fixed bin(31) value(13);
 dcl SNMP_BERAPI_INVALID_OBJELEM   fixed bin(31) value(14);
 dcl SNMP_PDUAPI_UNRECOGNIZED_PDU  fixed bin(31) value(20);
 dcl SNMP_PDUAPI_INVALID_ES        fixed bin(31) value(21);
 dcl SNMP_PDUAPI_INVALID_GT        fixed bin(31) value(22);
 dcl SNMP_AUTHAPI_INVALID_VERSION  fixed bin(31) value(30);
 dcl SNMP_AUTHAPI_INVALID_MSG_TYPE fixed bin(31) value(31);
 dcl SNMP_AUTHAPI_TRIV_AUTH_FAILED fixed bin(31) value(32);

 /*******************************************************************/
 /*                                                                 */
 /* Support for old definitions (support disabled via SNMPSTRICT)   */
 /*                                                                 */
 /*******************************************************************/

 %dcl SNMPSTRICT char ext;
 %if SNMPSTRICT = '' %then %do;

 %dcl SNMP_oidcpy char ext;
 %SNMP_oidcpy='SnmpUtilOidCpy';
 %dcl SNMP_oidappend char ext;
 %SNMP_oidappend='SnmpUtilOidAppend';
 %dcl SNMP_oidncmp char ext;
 %SNMP_oidncmp='SnmpUtilOidNCmp';
 %dcl SNMP_oidcmp char ext;
 %SNMP_oidcmp='SnmpUtilOidCmp';
 %dcl SNMP_oidfree char ext;
 %SNMP_oidfree='SnmpUtilOidFree';

 %dcl SNMP_CopyVarBindList char ext;
 %SNMP_CopyVarBindList='SnmpUtilVarBindListCpy';
 %dcl SNMP_FreeVarBindList char ext;
 %SNMP_FreeVarBindList='SnmpUtilVarBindListFree';
 %dcl SNMP_CopyVarBind char ext;
 %SNMP_CopyVarBind='SnmpUtilVarBindCpy';
 %dcl SNMP_FreeVarBind char ext;
 %SNMP_FreeVarBind='SnmpUtilVarBindFree';

 %dcl SNMP_printany char ext;
 %SNMP_printany='SnmpUtilPrintAsnAny';

 %dcl SNMP_free char ext;
 %SNMP_free='SnmpUtilMemFree';
 %dcl SNMP_malloc char ext;
 %SNMP_malloc='SnmpUtilMemAlloc';
 %dcl SNMP_realloc char ext;
 %SNMP_realloc='SnmpUtilMemReAlloc';

 %dcl SNMP_DBG_free char ext;
 %SNMP_DBG_free='SnmpUtilMemFree';
 %dcl SNMP_DBG_malloc char ext;
 %SNMP_DBG_malloc='SnmpUtilMemAlloc';
 %dcl SNMP_DBG_realloc char ext;
 %SNMP_DBG_realloc='SnmpUtilMemReAlloc';

 %dcl ASN_RFC1155_IPADDRESS char ext;
 %ASN_RFC1155_IPADDRESS='ASN_IPADDRESS';
 %dcl ASN_RFC1155_COUNTER char ext;
 %ASN_RFC1155_COUNTER='ASN_COUNTER32';
 %dcl ASN_RFC1155_GAUGE char ext;
 %ASN_RFC1155_GAUGE='ASN_GAUGE32';
 %dcl ASN_RFC1155_TIMETICKS char ext;
 %ASN_RFC1155_TIMETICKS='ASN_TIMETICKS';
 %dcl ASN_RFC1155_OPAQUE char ext;
 %ASN_RFC1155_OPAQUE='ASN_OPAQUE';
 %dcl ASN_RFC1213_DISPSTRING char ext;
 %ASN_RFC1213_DISPSTRING='ASN_OCTETSTRING';

 %dcl ASN_RFC1157_GETREQUEST char ext;
 %ASN_RFC1157_GETREQUEST='SNMP_PDU_GET';
 %dcl ASN_RFC1157_GETNEXTREQUEST char ext;
 %ASN_RFC1157_GETNEXTREQUEST='SNMP_PDU_GETNEXT';
 %dcl ASN_RFC1157_GETRESPONSE char ext;
 %ASN_RFC1157_GETRESPONSE='SNMP_PDU_RESPONSE';
 %dcl ASN_RFC1157_SETREQUEST char ext;
 %ASN_RFC1157_SETREQUEST='SNMP_PDU_SET';
 %dcl ASN_RFC1157_TRAP char ext;
 %ASN_RFC1157_TRAP='SNMP_PDU_V1TRAP';

 %dcl ASN_CONTEXTSPECIFIC char ext;
 %ASN_CONTEXTSPECIFIC='ASN_CONTEXT';
 %dcl ASN_PRIMATIVE char ext;
 %ASN_PRIMATIVE='ASN_PRIMITIVE';

 %dcl RFC1157VarBindList char ext;
 %RFC1157VarBindList='SnmpVarBindList';
 %dcl RFC1157VarBind char ext;
 %RFC1157VarBind='SnmpVarBind';
 %dcl AsnInteger char ext;
 %AsnInteger='AsnInteger32';
 %dcl AsnCounter char ext;
 %AsnCounter='AsnCounter32';
 %dcl AsnGauge char ext;
 %AsnGauge='AsnGauge32';

 %end; /* SNMPSTRICT */

 %end; /* _INC_SNMP */
